# -*- coding: utf-8 -*-

from django.urls import re_path

from watchman import views


urlpatterns = [
    re_path(r'^$', views.status, name="status"),
    re_path(r'^dashboard/$', views.dashboard, name="dashboard"),
    re_path(r'^ping/$', views.ping, name="ping"),
]



# -*- coding: utf-8 -*-

"""
test_django-watchman
------------

Tests for `django-watchman` utils module.
"""

from __future__ import unicode_literals

import unittest

from unittest.mock import patch

from watchman.utils import get_cache, get_checks


class TestWatchman(unittest.TestCase):

    def assert_lists_equal(self, list1, list2):
        try:
            # Python 3
            self.assertCountEqual(list1, list2)
        except AttributeError:
            # Python 2
            self.assertItemsEqual(list1, list2)

    @patch('watchman.utils.django_cache.caches', spec_set=dict)
    def test_get_cache_django_17_or_greater(self, get_cache_mock):
        get_cache('foo')
        get_cache_mock.__getitem__.called_once_with('foo')

    def test_get_checks_returns_all_available_checks_by_default(self):
        checks = [check.__name__ for check in get_checks()]
        expected_checks = ['caches', 'databases', 'storage']
        self.assert_lists_equal(checks, expected_checks)

    def test_get_checks_with_check_list_returns_union(self):
        check_list = ['watchman.checks.caches']
        checks = [check.__name__ for check in get_checks(check_list=check_list)]
        expected_checks = ['caches']
        self.assert_lists_equal(checks, expected_checks)

    def test_get_checks_with_skip_list_returns_difference(self):
        skip_list = ['watchman.checks.caches']
        checks = [check.__name__ for check in get_checks(skip_list=skip_list)]
        expected_checks = ['databases', 'storage']
        self.assert_lists_equal(checks, expected_checks)

    def test_get_checks_with_matching_check_and_skip_list_returns_empty_list(self):
        check_list, skip_list = ['watchman.checks.caches'], ['watchman.checks.caches']
        checks = [check.__name__ for check in get_checks(check_list=check_list, skip_list=skip_list)]
        expected_checks = []
        self.assert_lists_equal(checks, expected_checks)

    def test_get_checks_with_check_and_skip_list(self):
        check_list = ['watchman.checks.caches', 'watchman.checks.databases']
        skip_list = ['watchman.checks.caches']
        checks = [check.__name__ for check in get_checks(check_list=check_list, skip_list=skip_list)]
        expected_checks = ['databases']
        self.assert_lists_equal(checks, expected_checks)

    def test_get_checks_with_paid_checks_disabled_returns_expected_checks(self):
        expected_checks = ['caches', 'databases', 'storage']
        checks = [check.__name__ for check in get_checks()]
        self.assert_lists_equal(checks, expected_checks)

    @unittest.skip("Unsure how to test w/ modified settings")
    def test_get_checks_with_paid_checks_enabled_returns_expected_checks(self):
        pass



# -*- coding: utf-8 -*-

"""
test_django-watchman
------------

Tests for `django-watchman` management commands.
"""

from __future__ import unicode_literals

from io import StringIO
import unittest

from django.core.management import call_command


class TestWatchman(unittest.TestCase):

    def test_successful_management_command_outputs_nothing(self):
        out = StringIO()
        call_command('watchman', stdout=out)
        self.assertEquals('', out.getvalue())

    def test_successful_management_command_outputs_check_status_with_verbosity_2(self):
        out = StringIO()
        call_command('watchman', stdout=out, verbosity='2')
        self.assertIn('caches', out.getvalue())

    def test_successful_management_command_outputs_check_status_with_verbosity_3(self):
        out = StringIO()
        call_command('watchman', stdout=out, verbosity='3')
        self.assertIn('caches', out.getvalue())

    def test_successful_management_command_supports_check_list(self):
        out = StringIO()
        call_command('watchman', stdout=out, checks='watchman.checks.caches', verbosity='3')
        self.assertIn('caches', out.getvalue())
        self.assertNotIn('databases', out.getvalue())

    def test_successful_management_command_supports_skip_list(self):
        out = StringIO()
        call_command('watchman', stdout=out, skips='watchman.checks.email', verbosity='3')
        self.assertIn('caches', out.getvalue())
        self.assertIn('databases', out.getvalue())
        self.assertNotIn('email', out.getvalue())



# -*- coding: utf-8 -*-

from __future__ import unicode_literals
import warnings

from django.db.transaction import non_atomic_requests
from django.http import JsonResponse, HttpResponse
from django.shortcuts import render
from django.utils.translation import ugettext as _
from watchman import __version__, settings
from watchman.decorators import auth
from watchman.utils import get_checks


WATCHMAN_VERSION_HEADER = 'X-Watchman-Version'


def _get_check_params(request):
    check_list = None
    skip_list = None

    if len(request.GET) > 0:
        if 'check' in request.GET:
            check_list = request.GET.getlist('check')
        if 'skip' in request.GET:
            skip_list = request.GET.getlist('skip')

    return (check_list, skip_list)


def _deprecation_warnings():
    if settings.WATCHMAN_TOKEN:
        warnings.warn("`WATCHMAN_TOKEN` setting is deprecated, use `WATCHMAN_TOKENS` instead. It will be removed in django-watchman 1.0", DeprecationWarning)


def _disable_apm():
    # New Relic
    try:
        import newrelic.agent
        newrelic.agent.ignore_transaction(flag=True)
    except ImportError:
        pass
    # Datadog
    try:
        from ddtrace import tracer
        from ddtrace.constants import MANUAL_DROP_KEY
        tracer.current_span().set_tag(MANUAL_DROP_KEY)
    except (AttributeError, ImportError):
        pass


def run_checks(request):
    _deprecation_warnings()

    if settings.WATCHMAN_DISABLE_APM:
        _disable_apm()

    checks = {}
    ok = True

    check_list, skip_list = _get_check_params(request)

    for check in get_checks(check_list=check_list, skip_list=skip_list):
        if callable(check):
            _check = check()
            # Set our HTTP status code if there were any errors
            if settings.WATCHMAN_ERROR_CODE != 200:
                for _type in _check:
                    if type(_check[_type]) == dict:
                        result = _check[_type]
                        if not result['ok']:
                            ok = False
                    elif type(_check[_type]) == list:
                        for entry in _check[_type]:
                            for result in entry:
                                if not entry[result]['ok']:
                                    ok = False
            checks.update(_check)

    return checks, ok


@auth
@non_atomic_requests
def status(request):
    checks, ok = run_checks(request)

    if not checks:
        response = JsonResponse(
            {
                'error': 404,
                'message': _('No checks found'),
            },
            status=404
        )
    else:
        http_code = 200 if ok else settings.WATCHMAN_ERROR_CODE
        response = JsonResponse(checks, status=http_code)

    if settings.EXPOSE_WATCHMAN_VERSION:
        response[WATCHMAN_VERSION_HEADER] = __version__

    return response


@non_atomic_requests
def bare_status(request):
    checks, ok = run_checks(request)
    http_code = 200 if ok else settings.WATCHMAN_ERROR_CODE
    return HttpResponse(status=http_code, content_type='text/plain')


def ping(request):
    if settings.WATCHMAN_DISABLE_APM:
        _disable_apm()
    return HttpResponse('pong', content_type='text/plain')


@auth
@non_atomic_requests
def dashboard(request):
    checks, overall_status = run_checks(request)

    expanded_checks = {}
    for key, value in checks.items():
        if isinstance(value, dict):
            # For some systems (eg: email, storage) value is a
            # dictionary of status
            #
            # Example:
            # {
            #     'ok': True,  # Status
            # }
            #
            # Example:
            # {
            #     'ok': False,  # Status
            #     'error': "RuntimeError",
            #     'stacktrace': "...",
            # }
            single_status = value.copy()
            single_status['name'] = ''
            expanded_check = {
                'ok': value['ok'],
                'statuses': [single_status],
            }
        else:
            # For other systems (eg: cache, database) value is a
            # list of dictionaries of dictionaries of statuses
            #
            # Example:
            # [
            #     {
            #         'default': {  # Cache/database name
            #             'ok': True,  # Status
            #         }
            #     },
            #     {
            #         'non-default': {  # Cache/database name
            #             'ok': False,  # Status
            #             'error': "RuntimeError",
            #             'stacktrace': "...",
            #         }
            #     },
            # ]
            statuses = []
            for outer_status in value:
                for name, inner_status in outer_status.items():
                    detail = inner_status.copy()
                    detail['name'] = name
                    statuses.append(detail)

            expanded_check = {
                'ok': all(detail['ok'] for detail in statuses),
                'statuses': statuses,
            }
        expanded_checks[key] = expanded_check

    response = render(request, 'watchman/dashboard.html', {
        'checks': expanded_checks,
        'overall_status': overall_status,
        'watchman_version': __version__,
        'expose_watchman_version': settings.EXPOSE_WATCHMAN_VERSION,
    })

    if settings.EXPOSE_WATCHMAN_VERSION:
        response[WATCHMAN_VERSION_HEADER] = __version__

    return response



# -*- coding: utf-8 -*-

from django.core import cache as django_cache
from django.utils.module_loading import import_string

from watchman.settings import WATCHMAN_CHECKS


def get_cache(cache_name):
    return django_cache.caches[cache_name]


def get_checks(check_list=None, skip_list=None):
    checks_to_run = frozenset(WATCHMAN_CHECKS)

    if check_list is not None:
        checks_to_run = checks_to_run.intersection(check_list)
    if skip_list is not None:
        checks_to_run = checks_to_run.difference(skip_list)

    for python_path in checks_to_run:
        yield import_string(python_path)
